import re

from apiview.view import APIView
from django.views.decorators.http import require_http_methods

from xj_enroll.models import EnrollExtendField
from xj_enroll.service.subitem_service import SubitemService
from xj_user.utils.user_wrapper import user_authentication_wrapper
from ..utils.custom_response import util_response
from ..utils.model_handle import parse_data


def extend_filed_filter():
    try:
        extend_field_list = list(EnrollExtendField.objects.all().values("field_index", "alias_name"))
        extend_field_map = {i["field_index"]: i["alias_name"] for i in extend_field_list}
        return extend_field_map
    except Exception as e:
        print("扩展字段映射错误：", str(e))
        return {}


class SubitemApis(APIView):

    @require_http_methods(['GET'])
    @user_authentication_wrapper
    def list(self, *args, **kwargs, ):
        request_params = parse_data(self)
        data, err = SubitemService.list(params=request_params)
        extend_field_map = extend_filed_filter()
        result = []
        for item in data["list"]:
            tem_dict = {}
            for k, v in item.items():
                if not re.search("field_.*", k):
                    tem_dict[k] = v
                else:
                    alias = extend_field_map.get(k)
                    if alias:
                        tem_dict[alias] = v
            result.append(tem_dict)
        data["list"] = result
        if err:
            return util_response(err=1000, msg=data)
        return util_response(data=data)

    @require_http_methods(['POST'])
    def add(self, *args, **kwargs, ):
        params = parse_data(self)
        data, err = SubitemService.add(params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)



# encoding: utf-8
"""
@project: djangoModel->valuation_service
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 计价生成服务
@created_time: 2022/10/13 9:40
"""

# 接口服务类
from ..utils.j_valuation import JExpression


class ValuationService:
    expression = None  # 表达式

    @staticmethod
    def valuate(rule_id=None, enroll_id=None, ):
        # 测试代码
        # 案例代码
        # 1.解析括号找到对应的括号，意义对应关系
        # 2.递归解析公式
        # 5+IF(
        #     ((IF(1 = 5, 5, 0) + ((3))) >= (1 + 3 + 4)),
        #     "号外号外",
        #     IF(2 >= 60, "及格", "不及格")
        # ) + SUM(1,2,5,7,8)
        expression_string = JExpression.parse_variables(
            "5+if(((IF(1 = 5, 5, 0)+((3)))>= (1 + 3 + 4)),'号外号外',IF(2 >= 60, '及格', '不及格')) + SUM(1,2,5,7,8)",
            {"a": 1, "b": 2, "c": 5}
        )
        calculator = JExpression()
        return calculator.process(expression_string)

        # # 初始化字典
        # query_obj = Enroll.objects.filter(id=enroll_id)
        # if query_obj:
        #     enroll = query_obj.first().to_json()
        # else:
        #     enroll = {}

        # # 获取计价规则
        # rule_query_set = EnrollRuleValuate.objects.filter(id=rule_id)
        # if rule_query_set:
        #     expression_string = rule_query_set.value("expression_string").first()
        # else:
        #     return None, "没有找到计价公式"
        # expression_string = JExpression.parse_variables(expression_string, enroll) # 如果需要 table2.filed,仅仅需要对input_dict就行修改即可
        # return JExpression().process



"""
@project: djangoModel->tool
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: CURD 工具
@created_time: 2022/9/15 14:14
"""

from django.core.paginator import Paginator, EmptyPage

from ..models import Enroll, EnrollRecord
from ..utils.model_handle import format_params_handle


class EnrollServices:
    def __init__(self):
        pass

    @staticmethod
    def enroll_list(params):
        size = params.pop('size', 10)
        page = params.pop('page', 1)

        params['enroll_status_code'] = params['enroll_status_code'].split(";") if params.get("enroll_status_code") else None
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "id", "thread_id", "thread_id_list", "trading_relate", "region_code", "spend_time_start", "enroll_status_code",
                "spend_time_end", "create_time_start", "create_time_end", "finish_time_start", "finish_time_end",
                "open_time_start", "open_time_end", "enroll_status_code", "has_subitem"
            ],
            alias_dict={
                "spend_time_start": "spend_time__gte", "spend_time_end": "spend_time__lte", "create_time_start": "create_time__gte", "create_time_end": "create_time__lte",
                "finish_time_start": "finish_time__gte", "open_time_start": "open_time__gte", "open_time_end": "open_time__lte", "enroll_classify_value": "classify__value",
                "enroll_category_value": "category__value", "enroll_status_code": "enroll_status_code__in", "thread_id_list": "thread_id__in"
            }
        )
        enroll_obj = Enroll.objects.filter(**params).values()
        paginator = Paginator(enroll_obj, size)
        try:
            enroll_obj = paginator.page(page)
        except EmptyPage:
            enroll_obj = paginator.page(paginator.num_pages)
        except Exception as e:
            return None, f'{str(e)}'
        return {'total': paginator.count, "page": page, "size": size, 'list': list(enroll_obj.object_list)}, None

    @staticmethod
    def enroll_user_list(params):
        size = params.pop('size', 10)
        page = params.pop('page', 1)
        # is_finish = params.pop("is_finish", 0)
        # if is_finish:
        #     params['enroll_status_code__gte'] = 14
        # else:
        #     params['enroll_status_code__lt'] = 14
        enroll_obj = Enroll.objects.filter(**params).values()
        paginator = Paginator(enroll_obj, size)
        try:
            enroll_obj = paginator.page(page)
        except EmptyPage:
            enroll_obj = paginator.page(paginator.num_pages)
        except Exception as e:
            return None, f'{str(e)}'
        return {'total': paginator.count, "page": page, "size": size, 'list': list(enroll_obj.object_list)}, None

    @staticmethod
    def enroll_detail(enroll_id):
        enroll_obj = Enroll.objects.filter(id=enroll_id)
        if not enroll_obj:
            return None, "不存的报名信息"
        enroll_detail = enroll_obj.first().to_json()
        enroll_user_ids = list(EnrollRecord.objects.filter(enroll_id=enroll_id).values("user_id"))
        enroll_detail["enroll_user_ids"] = [i["user_id"] for i in enroll_user_ids]
        return enroll_detail, None

    @staticmethod
    def enroll_edit(params, enroll_id):
        enroll_id = params.pop("enroll_id", None) or enroll_id
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "thread_id", "trading_relate", "region_code", "occupy_room", "enroll_status_code", "min_number", "max_number", "min_count_apiece", "max_count_apiece",
                "enroll_rule_group", "price", "count", "unit", "fee", "reduction", "subitems_amount", "amount", "paid_amount", "unpaid_amount", "commision", "deposit",
                "bid_mode", "ticket", "hide_price", "hide_user", "has_repeat", "has_subitem", "has_audit", "has_vouch", "need_deposit", "need_imprest", "enable_pool",
                "pool_limit", "pool_stopwatch", "open_time", "close_time", "launch_time", "finish_time", "spend_time", "create_time", "update_time", "snapshot", "remark",
            ],
        )
        enroll_obj = Enroll.objects.filter(id=enroll_id)
        if not enroll_obj:
            return None, None
        try:
            enroll_obj.update(**params)
        except Exception as e:
            return None, "修改异常:" + str(e)
        return enroll_obj.first().to_json(), None

    @staticmethod
    def enroll_delete(enroll_id):
        enroll_obj = Enroll.objects.filter(id=enroll_id)
        if not enroll_obj:
            return None, None
        try:
            enroll_obj.delete()
        except Exception as e:
            return None, "删除异常:" + str(e)
        return None, None

    @staticmethod
    def enroll_add(params):
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "thread_id", "trading_relate", "region_code", "occupy_room", "enroll_status_code", "min_number", "max_number", "min_count_apiece", "max_count_apiece",
                "enroll_rule_group", "price", "count", "unit", "fee", "reduction", "subitems_amount", "amount", "paid_amount", "unpaid_amount", "commision", "deposit",
                "hide_price", "hide_user", "has_repeat", "has_subitem", "has_audit", "has_vouch", "need_deposit", "need_imprest", "enable_pool",
                "pool_limit", "pool_stopwatch", "open_time", "close_time", "launch_time", "finish_time", "spend_time", "create_time", "update_time", "snapshot", "remark",
            ],
        )
        try:
            instance = Enroll.objects.create(**params)
        except Exception as e:
            return None, str(e)

        return instance.to_json(), None



"""
Created on 2022-05-19
@author:刘飞
@description:报名模块序列化器
"""
from rest_framework import serializers
from .models import *


class EnrollListSerializer(serializers.ModelSerializer):
    """
    报名表序列化器
    """

    class Meta:
        model = Enroll
        fields = '__all__'


class EnrollRecordListSerializer(serializers.ModelSerializer):
    """
    报名记录列表序列化器
    """

    class Meta:
        model = EnrollRecord
        fields = '__all__'



from apiview.view import APIView
from django.db import transaction
from django.views.decorators.http import require_http_methods

from xj_enroll.service.subitem_service import SubitemService
from xj_thread.services.thread_item_service import ThreadItemService
from xj_thread.services.thread_list_service import ThreadListService
from xj_user.services.user_detail_info_service import DetailInfoService
from xj_user.utils.user_wrapper import user_authentication_force_wrapper, user_authentication_wrapper
from ..join_list import JoinList
from ..service.enroll_services import EnrollServices
from ..utils.custom_response import util_response
from ..utils.model_handle import parse_data, format_params_handle, request_params_wrapper


class EnrollAPI(APIView):

    @require_http_methods(['GET'])
    @request_params_wrapper
    @user_authentication_wrapper
    def list(self, *args, request_params=None, user_info=None, **kwargs, ):
        # 如果需要 信息表字典筛选，thread_id_list 放入thread_id即可
        data, err = EnrollServices.enroll_list(params=request_params)
        if err:
            return util_response(err=1000, msg=err)
        # 合并thread模块数据
        id_list = [i['thread_id'] for i in data['list'] if i]
        thread_list, err = ThreadListService.search(id_list)
        data['list'] = JoinList(data['list'], thread_list, "thread_id", "id").join()
        # 小数点四舍五入
        for i in data['list']:
            i["price"] = round(i["price"], 2)
        # 数据返回
        if err:
            return util_response(err=1001, msg=err)
        return util_response(data=data)

    @require_http_methods(['GET'])
    @request_params_wrapper
    @user_authentication_force_wrapper
    def own_list(self, *args, request_params=None, user_info=None, **kwargs):
        # is_finish = request_params.get("is_finish", 0)
        user_id = user_info.get("user_id")
        params = {"user_id": user_id, "page": request_params.get("page", 1), "size": request_params.get("page", 20)}
        data, err = EnrollServices.enroll_user_list(params)
        if err:
            return util_response(err=1000, msg=err)
        # 合并thread模块数据
        id_list = [i['thread_id'] for i in data['list'] if i]
        thread_list, err = ThreadListService.search(id_list)
        data['list'] = JoinList(data['list'], thread_list, "thread_id", "id").join()
        # 数据返回
        if err:
            return util_response(err=1001, msg=err)
        return util_response(data=data)

    @require_http_methods(['GET'])
    def detail(self, *args, **kwargs, ):
        params = parse_data(self) or {}
        enroll_id = kwargs.get("enroll_id") or params.pop("enroll_id") or None
        if not enroll_id:
            return util_response(err=1000, msg="参数错误:enroll_id不可以为空")
        data, err = EnrollServices.enroll_detail(enroll_id)
        # 获取已报名用户
        users = DetailInfoService.get_list_detail({}, data["enroll_user_ids"])
        data["enroll_user_info"] = [{"user_id": i["user_id"], 'nickname': i["nickname"], 'user_name': i["user_name"], 'full_name': i["full_name"]} for i in users]
        if err:
            return util_response(err=1001, msg=err)
        # 数据拼接
        thread_list, err = ThreadListService.search([data.get("thread_id")])
        if err:
            return util_response(err=1002, msg=err)
        data = JoinList([data], thread_list, "thread_id", "id").join()
        data = data[0] if not len(data) > 1 else data
        data["price"] = round(data["price"], 2)
        # 响应数据
        if err:
            return util_response(err=1003, msg=err)
        return util_response(data=data)

    @require_http_methods(['POST'])
    @user_authentication_force_wrapper
    def add(self, *args, user_info, **kwargs, ):
        user_id = user_info.get("user_id")
        params = parse_data(self)
        # 表单数据验证
        # is_valid, error = EnrollValidator(params).validate()
        # if not is_valid:
        #     return util_response(err=1000, msg=error)

        # 参数校验
        subitem_params = params.get("subitem")
        if not subitem_params:
            return util_response(err=1000, msg="分项表参数不能为空")
        thread_params = format_params_handle(
            param_dict=params,
            remove_filed_list=[
                # 报名表字段
                "trading_relate", "region_code", "occupy_room", "enroll_status_code", "min_number", "max_number",
                "min_count_apiece", "max_count_apiece", "enroll_rule_group", "price", "count", "unit", "fee", "reduction", "subitems_amount",
                "amount", "paid_amount", "unpaid_amount", "commision", "deposit", "hide_price", "hide_user", "has_repeat", "has_subitem", "has_audit",
                "need_vouch", "need_deposit", "need_imprest", "enable_pool", "pool_limit", "pool_stopwatch", "open_time", "close_time", "launch_time",
                "finish_time", "spend_time", "create_time", "update_time", "snapshot", "remark",
                # 报名分项字段
                "subitem"
            ],
            alias_dict={"thread_category_id": "category_id", "thread_classify_id": "classify_id", "thread_remark": "remark"}
        )
        if not thread_params:
            return util_response(err=1001, msg="请填写项目基本信息")

        main_params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "thread_id", "trading_relate", "region_code", "occupy_room", "enroll_status_code", "min_number", "max_number",
                "min_count_apiece", "max_count_apiece", "enroll_rule_group", "price", "count", "unit", "fee", "reduction", "subitems_amount",
                "amount", "paid_amount", "unpaid_amount", "commision", "deposit", "hide_price", "hide_user", "has_repeat", "has_subitem", "has_audit",
                "need_vouch", "need_deposit", "need_imprest", "enable_pool", "pool_limit", "pool_stopwatch", "open_time", "close_time", "launch_time",
                "finish_time", "spend_time", "create_time", "update_time", "snapshot", "remark"
            ]
        )
        sid = transaction.savepoint()

        # 信息表联调
        thread_params.setdefault("user_id", user_id)
        thread_id = thread_params.get("thread_id")
        if not thread_id:
            data, err = ThreadItemService.add(thread_params)
        else:
            data, err = ThreadItemService.edit(thread_params, thread_id)
        if err:
            transaction.savepoint_rollback(sid)
            return util_response(err=1002, msg="信息添加错误：" + err)

        thread_id = thread_id or data['id']
        main_params.setdefault("thread_id", thread_id or data['id'])
        # 报名主表添加
        main_params.setdefault("enroll_status_code", 0)
        main_params.setdefault("user_id", user_id)
        main_instance, err = EnrollServices.enroll_add(main_params)
        if err:
            transaction.savepoint_rollback(sid)
            return util_response(err=1003, msg=err)

        # 报名分项联动
        for item_params in params["subitem"]:
            item_params.setdefault("enroll_id", main_instance.get('id'))
            data, err = SubitemService.add(item_params)
            if err:
                transaction.savepoint_rollback(sid)
                return util_response(err=1004, msg=err)

        transaction.clean_savepoints()  # 清除保存点
        return util_response(data=main_instance)

    @require_http_methods(['PUT'])
    def edit(self, *args, **kwargs, ):
        params = parse_data(self)
        enroll_id = kwargs.get("enroll_id") or params.pop("enroll_id") or None
        if not enroll_id:
            return util_response(err=1000, msg="参数错误:enroll_id不可以为空")
        data, err = EnrollServices.enroll_edit(params, enroll_id)
        if err:
            return util_response(err=1000, msg=err)
        # TODO联动信息模块修改
        thread_id = params.pop("thread_id", None) or data.get("thread_id", None)
        if thread_id:
            thread_params = format_params_handle(
                param_dict=params,
                remove_filed_list=[
                    "thread_id", "trading_relate", "region_code", "occupy_room", "enroll_status_code", "min_number", "max_number",
                    "min_count_apiece", "max_count_apiece", "enroll_rule_group", "price", "count", "unit", "fee", "reduction", "subitems_amount",
                    "amount", "paid_amount", "unpaid_amount", "commision", "deposit", "hide_price", "hide_user", "has_repeat", "has_subitem", "has_audit",
                    "need_vouch", "need_deposit", "need_imprest", "enable_pool", "pool_limit", "pool_stopwatch", "open_time", "close_time", "launch_time",
                    "finish_time", "spend_time", "create_time", "update_time", "snapshot", "remark"
                ]
            )
            if thread_params:
                data, err = ThreadItemService.edit(thread_params, thread_id)
                if err:
                    return util_response(err=1001, msg=err)

        return util_response(data=data)

    @require_http_methods(['DELETE'])
    def delete(self, *args, **kwargs, ):
        params = parse_data(self)
        enroll_id = kwargs.get("enroll_id") or params.pop("enroll_id") or None
        if not enroll_id:
            return util_response(err=1000, msg="参数错误:enroll_id不可以为空")
        data, err = EnrollServices.enroll_delete(enroll_id)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)



# encoding: utf-8
"""
@project: djangoModel->j_subitem_extend
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 报名分项扩展字段映射工具
@created_time: 2022/10/16 13:26
"""
import re

from ..models import EnrollSubitemExtendField


# 扩展字段映射
def extend_filed_map(category_id):
    if not category_id:
        return [], {}

    try:
        query_obj = EnrollSubitemExtendField.objects.filter(category_id=category_id)
        if not query_obj:
            return [], {}
        extend_field_list = list(query_obj.values("field_index", "field"))
        extend_field_map = {i["field"]: i["field_index"] for i in extend_field_list}
        filter_filed_list = list(extend_field_map.keys())
        return filter_filed_list, extend_field_map

    except Exception as e:
        print("扩展字段映射错误：", str(e))
        return [], {}


def filter_field_index(result_list=[], result_dict={}):
    extend_field_list = list(EnrollSubitemExtendField.objects.all().values("category_id", "field_index", "field"))
    extend_field_map = {}
    for item in extend_field_list:
        category_id = extend_field_map.get(item["category_id"])
        if not category_id:
            extend_field_map[item["category_id"]] = {item["field"]: item["field_index"]}
        else:
            extend_field_map[item["category_id"]].update({item["field"]: item["field_index"]})

    for res_item in result_list:
        enroll_id = res_item
        for item_item in res_item:
            if not re.search("field_\d*/", item_item):
                continue



from django.urls import re_path

from xj_enroll.api.subitem_apis import SubitemApis
from .api.enroll_apis import EnrollAPI
from .api.rarely_data_list_api import OtherListAPIView
from .api.record_apis import RecordAPI
from .api.rule_apis import RuleAPI
from .api.valuation_api import ValuationAPIView

urlpatterns = [
    # 报名API
    re_path(r'^list/?$', EnrollAPI.list),
    re_path(r'^own_list/?$', EnrollAPI.own_list),
    re_path(r'^detail/?(?P<enroll_id>\d+)?$', EnrollAPI.detail),
    re_path(r'^edit/?(?P<enroll_id>\d+)?$', EnrollAPI.edit),
    re_path(r'^delete/?(?P<enroll_id>\d+)?$', EnrollAPI.delete),
    re_path(r'^add/?$', EnrollAPI.add),
    re_path(r'^subitem_add/?$', SubitemApis.add),
    re_path(r'^subitem_list/?$', SubitemApis.list),

    # re_path(r'^category_list/?$', CategoryApi.list),
    # re_path(r'^category_edit/?(?P<category_id>\d+)?$', CategoryApi.edit),
    # re_path(r'^category_delete/?(?P<category_id>\d+)?$', CategoryApi.delete),
    # re_path(r'^category_add/?$', CategoryApi.add),
    #
    # re_path(r'^classify_list/?$', ClassifyApi.list),
    # re_path(r'^classify_edit/?(?P<classify_id>\d+)?$', ClassifyApi.edit),
    # re_path(r'^classify_delete/?(?P<classify_id>\d+)?$', ClassifyApi.delete),
    # re_path(r'^classify_add/?$', ClassifyApi.add),

    # 报名规则
    re_path(r'^rule_list/?$', RuleAPI.list),
    re_path(r'^rule_edit/?(?P<rule_value_id>\d+)?$', RuleAPI.edit),
    re_path(r'^rule_delete/?(?P<rule_value_id>\d+)?$', RuleAPI.delete),
    re_path(r'^rule_add/?$', RuleAPI.add),
    re_path(r'^rule_group_list/?$', RuleAPI.group_list),  # 分组列表

    # 报名记录
    re_path(r'^record_list/?$', RecordAPI.list),
    re_path(r'^record_add/?$', RecordAPI.add),
    re_path(r'^record_del/?(?P<pk>\d+)?$', RecordAPI.record_del),
    re_path(r'^record_edit/?(?P<pk>\d+)?$', RecordAPI.record_edit),

    # 计价接口
    re_path(r'^valuation_item/?$', ValuationAPIView.list),

    # 非热更新列表返回
    re_path(r'^status_code/?$', OtherListAPIView.enroll_status_code),

]



from django.views.decorators.http import require_http_methods
from rest_framework.views import APIView

from xj_user.services.user_detail_info_service import DetailInfoService
from xj_user.utils.user_wrapper import user_authentication_force_wrapper
from ..service.enroll_record_serivce import EnrollRecordServices
from ..utils.custom_response import util_response
from ..utils.join_list import JoinList
from ..utils.model_handle import parse_data


class RecordAPI(APIView):
    # 添加记录,用户报名
    @require_http_methods(['POST'])
    @user_authentication_force_wrapper
    def add(self, *args, user_info=None, **kwargs, ):
        params = parse_data(self) or {}
        params['user_id'] = user_info.get("user_id")
        # 表单数据验证
        # is_valid, error = RecordValidator(params).validate()
        # if not is_valid:
        #     return util_response(err=1000, msg=error)
        # 添加数据
        data, err = EnrollRecordServices.record_add(params)
        if err:
            return util_response(err=1001, msg=err)
        return util_response(data=data)

    @require_http_methods(['GET'])
    def list(self, *args, **kwargs, ):
        params = parse_data(self)
        data, err = EnrollRecordServices.record_list(params=params)
        user_ids = []
        for i in data["list"]:
            i["fee"] = round(i["fee"], 2)
            i["price"] = round(i["price"], 2)
            i["amount"] = round(i["amount"], 2)
            i["deposit_amount"] = round(i["deposit_amount"], 2)
            i["coupon_amount"] = round(i["coupon_amount"], 2)
            i["again_reduction"] = round(i["again_reduction"], 2)
            i["subitems_amount"] = round(i["subitems_amount"], 2)
            i["paid_amount"] = round(i["paid_amount"], 2)
            i["unpaid_amount"] = round(i["unpaid_amount"], 2)
            i["main_amount"] = round(i["main_amount"], 2)
            i["deposit"] = round(i["deposit"], 2)
            i["count"] = round(i["count"], 0)
            user_ids.append(i["user_id"])

        user_infos = DetailInfoService.get_list_detail({}, user_ids)
        data["list"] = JoinList(data["list"], user_infos, "user_id", "user_id").join()
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @require_http_methods(['DELETE'])
    def record_del(self, *args, **kwargs, ):
        params = parse_data(self) or {}
        pk = kwargs.get("pk") or params.pop("id")
        data, err = EnrollRecordServices.record_del(pk)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @require_http_methods(['PUT'])
    def record_edit(self, *args, **kwargs, ):
        params = parse_data(self) or {}
        pk = kwargs.get("pk") or params.pop("id")
        data, err = EnrollRecordServices.record_edit(params, pk)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)



"""
Created on 2022-01-17
@author:刘飞
@description:自定义返回格式
"""
from django.http import JsonResponse
from rest_framework import status


# 数据返回规则
def util_response(data='', err=0, msg='ok', http_status=status.HTTP_200_OK):
    if http_status == status.HTTP_200_OK:
        return JsonResponse({'err': err, 'msg': msg, 'data': data or {}})
    else:
        return JsonResponse({'err': http_status, 'msg': msg, }, status=http_status)


# 自定义错误页面时返回【DEBUG为True时生效】
def bad_request(request, exception):
    data = {'err': status.HTTP_400_BAD_REQUEST, 'msg': '参数错误', 'data': None}
    return JsonResponse(data=data, status=status.HTTP_400_BAD_REQUEST)


def permission_denied(request, exception):
    data = {'err': status.HTTP_403_FORBIDDEN, 'msg': '无权限', 'data': None}
    return JsonResponse(data=data, status=status.HTTP_403_FORBIDDEN)


def page_not_found(request, exception):
    data = {'err': status.HTTP_404_NOT_FOUND, 'msg': '资源不存在', 'data': None}
    return JsonResponse(data=data, status=status.HTTP_404_NOT_FOUND)


def page_error(exception):
    data = {'err': status.HTTP_500_INTERNAL_SERVER_ERROR, 'msg': '服务器错误', 'data': None}
    return JsonResponse(data=data, status=status.HTTP_500_INTERNAL_SERVER_ERROR)



# encoding: utf-8
"""
@project: djangoModel->subitem_service
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 报名分项记录
@created_time: 2022/10/15 12:38
"""
from django.core.paginator import Paginator

from ..models import EnrollSubitem, EnrollExtendField
from ..utils.model_handle import format_params_handle


def extend_filed_filter():
    try:
        extend_field_list = list(EnrollExtendField.objects.all().values("field_index", "alias_name"))
        extend_field_map = {i["alias_name"]: i["field_index"] for i in extend_field_list}
        filter_filed_list = list(extend_field_map.keys())
        return filter_filed_list, extend_field_map
    except Exception as e:
        print("扩展字段映射错误：", str(e))
        return [], {}


class SubitemService:

    @staticmethod
    def list(params):
        size = params.pop('size', 10)
        page = params.pop('page', 1)
        # 字段过滤
        filter_filed_list, extend_field_map = extend_filed_filter()
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=["id", "enroll_id", "name", "price", "count", "unit", "description", "remark"] + filter_filed_list,
            alias_dict=extend_field_map
        )

        try:
            fetch_obj = EnrollSubitem.objects.filter(**params).values()
            paginator = Paginator(fetch_obj, size)
            page_obj = paginator.page(page)
            result_list = list(page_obj.object_list)

            data = {'total': paginator.count, "size": size, 'page': page, 'list': result_list}
            return data, None
        except Exception as e:
            return None, str(e)

    @staticmethod
    def edit(params, subitem_rule_id):
        subitem_rule_id = params.pop("id", None) or subitem_rule_id

        filter_filed_list, extend_field_map = extend_filed_filter()
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=["id", "enroll_id", "name", "price", "count", "unit", "description", "remark"] + filter_filed_list,
            alias_dict=extend_field_map
        )

        subitem_enroll_obj = EnrollSubitem.objects.filter(id=subitem_rule_id)
        if not subitem_enroll_obj:
            return None, None
        try:
            subitem_enroll_obj.update(**params)
        except Exception as e:
            return None, "修改异常:" + str(e)
        return None, None

    @staticmethod
    def delete(subitem_rule_id):
        subitem_enroll_obj = EnrollSubitem.objects.filter(id=subitem_rule_id)
        if not subitem_enroll_obj:
            return None, None
        try:
            subitem_enroll_obj.delete()
        except Exception as e:
            return None, "删除异常:" + str(e)
        return None, None

    @staticmethod
    def add(params):
        filter_filed_list, extend_field_map = extend_filed_filter()
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=["enroll_id", "name", "price", "count", "unit", "description", "remark"] + filter_filed_list,
            alias_dict=extend_field_map
        )
        if not params.get("enroll_id"):
            return None, "请填写报名ID"
        try:
            EnrollSubitem.objects.create(**params)
        except Exception as e:
            return None, str(e)

        return None, None



# encoding: utf-8
"""
@project: djangoModel->j_valuation
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 计价服务类
@created_time: 2022/10/13 16:58
"""
import re


# 计算器基类
class JBaseExpression:
    __result = None  # 结果
    __expression = None
    fun_type = ""

    # 获取所有子类
    @staticmethod
    def get_child_info():
        try:
            return {getattr(i, "name"): i for i in JBaseExpression.__subclasses__() if getattr(i, "name", None)}, None
        except AttributeError:
            return None, "扩展类name属性可以重复"

    # 解析变量
    @staticmethod
    def parse_variables(expression_string, input_dict):
        if expression_string is None:
            return 0
        # 变量解析替换
        this_cell_value_match = re.compile("{{.*?}}").findall(str(expression_string))
        # 得到变量解析的键值对
        parsed_variable_map = {}
        for i in this_cell_value_match:
            parsed_variable_map.update({i: input_dict.get(i.replace("{{", "").replace("}}", ""), 0)})
        # 比那辆键值对替换
        for k, v in parsed_variable_map.items():
            expression_string = expression_string.replace(k, str(v))
        return expression_string

    # 解析括号，成对解析
    @staticmethod
    def parsed_brackets(expression):
        twain_index_map = {}
        forward_index_list = twain_index_list = []
        for char, index in zip(expression, range(len(expression))):
            if char == "(":
                forward_index_list.append(index)
            if char == ")":
                forward_bracket = forward_index_list.pop(-1)
                twain_index_map[forward_bracket] = (forward_bracket, index)

        return twain_index_list, twain_index_map


# 计算器
class JExpression(JBaseExpression):
    def process(self, expression_string):
        if expression_string is None:
            return "", None
        if not len(re.findall("\(", expression_string)) == len(re.findall("\)", expression_string)):
            return None, "语法错误，括号应该成对存在"
        child_class, err = self.get_child_info()
        if err:
            return None, err
        # 全部转换大写
        expression_string = expression_string.upper().replace(" ", "").replace("=", "==")
        # 解析括号和函数
        brackets_twain_index, brackets_twain_map = self.parsed_brackets(expression_string)
        # for name, instance in child_class.items():
        #     for r in re.finditer(name, expression_string):
        #         name_start, name_end = r.span()
        #         bracket_start, bracket_end = brackets_twain_map[name_end]
        #         fun_str = expression_string[name_start:bracket_end + 1]
        #         result_str = instance().process(fun_str)
        # result = CalculateExpression().process(expression_string)
        # 运行公式
        # return result, None
        return None, None


# ================================= 公式封装类 =======================================
#  基础计算
class CalculateExpression(JBaseExpression):

    def process(self, expression_string):
        try:
            return eval(expression_string), None  # 加加减乘除
        except ZeroDivisionError:
            return 0, None  # 0/n  的情况
        except Exception as e:
            return expression_string, str(e)


# 布尔值判断、三元运算
class IFExpression(JBaseExpression):
    name = "IF"
    patt = "^IF\((.*?),(.*?),(.*?)\)&"

    # if(((if( a==5,5,0)+((3))) >= (1+3+4)), '号外号外', if(x>=60, '及格', '不及格'))

    def resolve(self, expression):
        pass

    def process(self, expression, *args, ):
        bool_expression, yes_expression, no_expression = expression.split(",")
        if bool_expression:
            return yes_expression
        else:
            return no_expression


# 求和计算、（多参数在服务中获取）
class SUMExpression(JBaseExpression):
    name = "SUM"
    patt = "SUM(.*?)"  # SUM(a,b,c) == a+b+c

    def process(self, expression):
        args = expression.split(",")
        result = 0
        for i in args:
            result += i
        return result



# encoding: utf-8
"""
@project: djangoModel->valuation_api
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis:
@created_time: 2022/10/13 11:22
"""
from rest_framework.views import APIView

from ..service.valuation_service import ValuationService
from ..utils.custom_response import util_response
from ..utils.model_handle import request_params_wrapper


class ValuationAPIView(APIView):
    # 获取计价
    @request_params_wrapper
    def list(self, *args, request_params, **kwargs):
        print("request_params:", request_params)
        data, err = ValuationService.valuate()
        if err:
            return util_response(err=1000, msg=err)
        return util_response()
